/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIRequestObserver.idl"

interface nsIEventTarget;

/**
 * A request observer proxy is used to ship data over to another thread
 * specified by the thread's dispatch target. The "true" request observer's
 * methods are invoked on the other thread.
 *
 * This interface only provides the initialization needed after construction.
 * Otherwise, these objects are used simply as nsIRequestObserver's.
 */
[scriptable, uuid(7df8845f-938a-4437-9ea4-b11b850048f1)]
interface nsIRequestObserverProxy : nsIRequestObserver
{
    /**
     * Initializes an nsIRequestObserverProxy.
     *
     * @param observer - receives observer notifications on the other thread
     * @param target - may be NULL indicating the calling thread's event target
     */
    void init(in nsIRequestObserver observer,
              in nsIEventTarget target);
};
