/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMUIEvent.idl"

/**
 * For more information on this interface please see
 * http://www.w3.org/TR/SVG11/script.html#InterfaceSVGZoomEvent
 */

interface nsIDOMSVGRect;

[scriptable, builtinclass, uuid(9cf032c6-8485-4bde-90d4-0c8bbff27a12)]
interface nsIDOMSVGZoomEvent : nsIDOMUIEvent
{
  // Not implemented
  // readonly attribute nsIDOMSVGRect  zoomRectScreen;
  readonly attribute float          previousScale;
  // SVGPoint
  readonly attribute nsISupports    previousTranslate;
  readonly attribute float          newScale;
  // SVGPoint
  readonly attribute nsISupports    newTranslate;
};
