/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM /builds/slave/m-cen-lx-xr-ntly-0000000000000/build/dom/file/nsIDOMArchiveRequest.idl
 */

#ifndef __gen_nsIDOMArchiveRequest_h__
#define __gen_nsIDOMArchiveRequest_h__


#ifndef __gen_nsIDOMDOMRequest_h__
#include "nsIDOMDOMRequest.h"
#endif

/* For IDL files that don't want to include root IDL files. */
#ifndef NS_NO_VTABLE
#define NS_NO_VTABLE
#endif
class nsIDOMArchiveReader; /* forward declaration */


/* starting interface:    nsIDOMArchiveRequest */
#define NS_IDOMARCHIVEREQUEST_IID_STR "cb72a719-f07d-4271-a120-16e9148a8699"

#define NS_IDOMARCHIVEREQUEST_IID \
  {0xcb72a719, 0xf07d, 0x4271, \
    { 0xa1, 0x20, 0x16, 0xe9, 0x14, 0x8a, 0x86, 0x99 }}

class NS_NO_VTABLE nsIDOMArchiveRequest : public nsIDOMDOMRequest {
 public: 

  NS_DECLARE_STATIC_IID_ACCESSOR(NS_IDOMARCHIVEREQUEST_IID)

  /* readonly attribute nsISupports reader; */
  NS_IMETHOD GetReader(nsISupports * *aReader) = 0;

};

  NS_DEFINE_STATIC_IID_ACCESSOR(nsIDOMArchiveRequest, NS_IDOMARCHIVEREQUEST_IID)

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIDOMARCHIVEREQUEST \
  NS_IMETHOD GetReader(nsISupports * *aReader); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIDOMARCHIVEREQUEST(_to) \
  NS_IMETHOD GetReader(nsISupports * *aReader) { return _to GetReader(aReader); } 

/* Use this macro to declare functions that forward the behavior of this interface to another object in a safe way. */
#define NS_FORWARD_SAFE_NSIDOMARCHIVEREQUEST(_to) \
  NS_IMETHOD GetReader(nsISupports * *aReader) { return !_to ? NS_ERROR_NULL_POINTER : _to->GetReader(aReader); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsDOMArchiveRequest : public nsIDOMArchiveRequest
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIDOMARCHIVEREQUEST

  nsDOMArchiveRequest();

private:
  ~nsDOMArchiveRequest();

protected:
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsDOMArchiveRequest, nsIDOMArchiveRequest)

nsDOMArchiveRequest::nsDOMArchiveRequest()
{
  /* member initializers and constructor code */
}

nsDOMArchiveRequest::~nsDOMArchiveRequest()
{
  /* destructor code */
}

/* readonly attribute nsISupports reader; */
NS_IMETHODIMP nsDOMArchiveRequest::GetReader(nsISupports * *aReader)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIDOMArchiveRequest_h__ */
