/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMXULElement.idl"
#include "nsIDOMElement.idl"

interface nsITreeColumns;
interface nsITreeView;
interface nsIDOMXULTextBoxElement;

/**
 * @status UNDER_DEVELOPMENT
 */

[scriptable, uuid(e582a355-803f-4b68-93d0-9396037cc98e)]
interface nsIDOMXULTreeElement : nsIDOMXULElement
{

  readonly attribute nsITreeColumns columns;

  attribute nsITreeView view;

  readonly attribute nsIDOMElement body;

  attribute boolean editable;

  // For editable trees only.
  readonly attribute nsIDOMXULTextBoxElement inputField;

};
