/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"
#include "nsIDOMDOMRequest.idl"

/**
  * The constructor will accept a nsIDOMActivityOptions parameter.
  */
[scriptable, builtinclass, uuid(1f59cd9a-b8b5-4a98-bd2a-897b73584946)]
interface nsIDOMMozActivity : nsIDOMDOMRequest
{
};
