/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"


%{C++

#define NS_AUTOCONFIG_CID\
  { 0xe036c738,\
    0x1dd1,\
    0x11b2,\
    { 0x93, 0x92, 0x9d, 0x94, 0xaa, 0x74, 0xb0, 0xc5 }\
  }
 
#define NS_AUTOCONFIG_CONTRACTID \
  "@mozilla.org/autoconfiguration;1" 

#define NS_AUTOCONFIG_CLASSNAME "AutoConfig Module"

%}

[uuid (80DB54AE-13F2-11d5-BE44-00108335A220)]
interface nsIAutoConfig : nsISupports {
  attribute string configURL;
};
