/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim:set ts=2 sw=2 sts=2 et cindent: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
#include "domstubs.idl"

[scriptable, uuid(c43448db-0bab-461d-b648-1ca14a967f7e)]
interface nsIDOMTimeRanges : nsISupports
{
  /* The number of ranges represented by the time range object */
  readonly attribute unsigned long length;

  /* The position of the start of the index'd range, in seconds measured
     from the start of the timeline that this object represents */
  double start(in unsigned long index);

  /* The position of the end of the index'd range, in seconds measured
     from the start of the timeline that this object represents */
  double end(in unsigned long index);
};
