/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface nsIDOMSVGRect;

[scriptable, uuid(ca45959e-f1da-46f6-af19-1ecdc322285a)]
interface nsIDOMSVGAnimatedRect : nsISupports
{
  readonly attribute nsIDOMSVGRect baseVal;
  readonly attribute nsIDOMSVGRect animVal;
};
