/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMText.idl"

/**
 * CDATA sections are used to escape blocks of text containing characters 
 * that would otherwise be regarded as markup.
 * Their primary purpose is for including material such as XML fragments, 
 * without needing to escape all the delimiters.
 * 
 * For more information on this interface please see 
 * http://www.w3.org/TR/DOM-Level-2-Core/
 */

[scriptable, uuid(6e2f1af8-cc86-4ed5-948a-007bb40c65da)]
interface nsIDOMCDATASection : nsIDOMText
{
};
